#!/usr/bin/pyhton
from RciLog import RciLog
from array import array
from time import time
from struct import pack_into,unpack_from

class RciPacket():
	
	# RCI Start packet magic number
	RCI_MAGIC_NUMBER=0x1919A555
	# RCI protocol Revision
	RCI_MAJOR_VERSION=0x00
	RCI_MINOR_VERSION=0x02
	# RCI Max payload size
	RCI_MAX_NUM_BYTES_PAYLOAD = 1024 * 1024 * 4 # TBC: Max. 4 MBytes per packet
	# RCI packet type IDS
	RCI_PID = { 
				'RCI_REQUEST_PID'	: 0x00,
				'RCI_REPLY_PID'		: 0x01,
			}

	SIZEOF_ULONG=8
	FORMAT_ULONG='!q'
	SIZEOF_UINT=4
	FORMAT_UINT='!I'
	SIZEOF_USHORT=2
	FORMAT_USHORT='!H'
	SIZEOF_UBYTE=1
	FORMAT_UBYTE='!B'
	RCI_MIN_NUM_BYTES_PACKET = 3 * SIZEOF_UINT + ( 3 *SIZEOF_UBYTE ) +  SIZEOF_ULONG
	
	log = RciLog("RciPacket")

	def __init__(self, pidKey):
		self.magicNumber  = self.RCI_MAGIC_NUMBER
		self.majorVersion = self.RCI_MAJOR_VERSION
		self.minorVersion = self.RCI_MINOR_VERSION
		pids = self.RCI_PID.viewkeys()
		if pidKey in pids:
			self.pid = self.RCI_PID[pidKey]
			self.log.write("Create Rci Packet with pid value " + pidKey, 'NOTIFICATION')
		else:
			raise ValueError("Unsupported Value of PID " + pidKey)
		self.timeStamp			= time()*1000
		self.packetDescriptor	= 0 
		self.numBytesPayload	= 0
		self.payload=array('c')

	# Handle Magic Number
	def getMagicNumber(self):
		return self.magicNumber
		
	def setMagicNumber(self,m):
		self.magicNumber = m
	
	def isStartOfFrameCorrect(self):
		return self.magicNumber == self.RCI_MAGIC_NUMBER

	# Handle Major Version
	def getMajorVersion(self):
		return self.majorVersion
		
	def setMajorVersion(self,m):
		self.majorVersion = m

	def isMajorVersionSupported(self):
		return self.majorVersion == self.RCI_MAJOR_VERSION

	# Handle Minor Version
	def getMinorVersion(self):
		return self.minorVersion
		
	def setMinorVersion(self,m):
		self.minorVersion = m	

	def isMinorVersionSupported(self):
		return self.minorVersion <= self.RCI_MINOR_VERSION
	
	# Handle Packet Type ID
	def getPacketID(self):
		return self.pid
	
	def setPacketID(self,pid):
		self.pid = pid

	def isRequestPacket(self):
		return self.pid == self.RCI_PID["RCI_REQUEST_PID"]
		
	def isReplyPacket(self):
		return self.pid == self.RCI_PID["RCI_REPLY_PID"]
	
	# Handle Packet descriptor
	def setPacketDescriptor(self,d):
		self.packetDescriptor = d
	
	def getPacketDescriptor(self):
		return self.packetDescriptor
	
	# Handle Timestamp	
	def setCurrentTimeStamp(self):
			self.timeStamp=time()*1000

	def setTimeStamp(self,t):
			self.timeStamp=t
				
	def getTimeStamp(self):
		return self.timeStamp
	
	# Handle Num Bytes Payload
	def getNumBytesPayload(self):
		return self.numBytesPayload

	def setNumBytesPayload(self,numBytesPayload):
		try:
			if self.numBytesPayload > self.RCI_MAX_NUM_BYTES_PAYLOAD :
				raise ValueError ("Too many payload bytes")
			self.numBytesPayload = numBytesPayload
		except ValueError as v:
			raise v

	# Handle Payload
	def setPayload(self,p):
		self.payload = p 
		
	def getPayload(self):
		if self.numBytesPayload == 0 :
			return None
		return self.payload

	# Write an RCI packet into output buffer
	def packetWrite(self,outputBuffer):	
		
		offset=0

		self.log.write("packetWrite : MagicNumber " + str(self.getMagicNumber()),'DEBUG')
		pack_into(self.FORMAT_UINT,outputBuffer,offset,self.getMagicNumber())
		offset+=self.SIZEOF_UINT

		self.log.write("packetWrite : MajorVersion " + str(self.getMajorVersion()),'DEBUG')
		pack_into(self.FORMAT_UBYTE,outputBuffer,offset,self.getMajorVersion())
		offset+=self.SIZEOF_UBYTE

		self.log.write("packetWrite : MinorVersion " + str(self.getMinorVersion()),'DEBUG')
		pack_into(self.FORMAT_UBYTE,outputBuffer,offset,self.getMinorVersion())
		offset+=self.SIZEOF_UBYTE
					
		self.log.write("packetWrite : PacketID " + str(self.getPacketID()),'DEBUG')
		pack_into(self.FORMAT_UBYTE,outputBuffer,offset,self.getPacketID())
		offset+=self.SIZEOF_UBYTE
		
		self.log.write("packetWrite : PacketDescriptor " + str(self.getPacketDescriptor()),'DEBUG')
		pack_into(self.FORMAT_UINT,outputBuffer,offset,self.getPacketDescriptor())
		offset+=self.SIZEOF_UINT

		self.log.write("packetWrite : TimeStamp " + str(self.getTimeStamp()),'DEBUG')
		pack_into(self.FORMAT_ULONG,outputBuffer,offset,self.getTimeStamp())
		offset+=self.SIZEOF_ULONG
		
		self.log.write("packetWrite : NumBytesPayload " + str(self.getNumBytesPayload()),'DEBUG')
		pack_into(self.FORMAT_UINT,outputBuffer,offset,self.getNumBytesPayload())
		offset+=self.SIZEOF_UINT
		
		# Copy Payload byte by byte
		if self.getNumBytesPayload() != 0:
			for idx in range(self.getNumBytesPayload()):
				outputBuffer.append(self.payload[idx])

		self.log.write("outputBuffer size = " + str(len(outputBuffer)),'DEBUG')


	# Read an RCI packet from input buffer
	def packetRead(self,inputBuffer):
		try:
			# Control that received packet contains at least necessary bytes fro a reply packet
			nBytes=len(inputBuffer)
			self.log.write("packetRead : nBytes of inputBuffer " + str(nBytes),'DEBUG')
			if nBytes < self.RCI_MIN_NUM_BYTES_PACKET :
				raise ValueError("Minimal size of reply packet is not respected")
			# Retrieve reply packet fields 
			offset=0
			self.setMagicNumber(unpack_from(self.FORMAT_UINT,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UINT
			self.log.write("packetRead : MagicNumber " + str(self.getMagicNumber()),'DEBUG')
			
			self.setMajorVersion(unpack_from(self.FORMAT_UBYTE,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UBYTE
			self.log.write("packetRead : MajorVersion " + str(self.getMajorVersion()),'DEBUG')
			
			self.setMinorVersion(unpack_from(self.FORMAT_UBYTE,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UBYTE
			self.log.write("packetRead : MinorVersion " + str(self.getMinorVersion()),'DEBUG')
			
			self.setPacketID(unpack_from(self.FORMAT_UBYTE,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UBYTE
			self.log.write("packetRead : PacketID " + str(self.getPacketID()),'DEBUG')
			
			self.setPacketDescriptor(unpack_from(self.FORMAT_UINT,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UINT
			self.log.write("packetRead : PacketDescriptor " + str(self.getPacketDescriptor()),'DEBUG')

			self.setTimeStamp(unpack_from(self.FORMAT_ULONG,inputBuffer,offset)[0])
			offset+=self.SIZEOF_ULONG
			self.log.write("packetRead : TimeStamp " + str(self.getTimeStamp()),'DEBUG')

			self.setNumBytesPayload(unpack_from(self.FORMAT_UINT,inputBuffer,offset)[0])
			offset+=self.SIZEOF_UINT
			self.log.write("packetRead : NumBytesPayload " + str(self.getNumBytesPayload()),'DEBUG')

			# Copy Payload byte by byte
			if self.getNumBytesPayload() != 0:
				self.log.write("packetRead : Start the set of payload from offset " + str(offset),'DEBUG')
				for idx in range(self.getNumBytesPayload()):
					self.payload.append(inputBuffer[offset+idx])

		except ValueError as e:
			self.log.write("packetRead : Catch exception:",'ERROR')
			raise e

	# Clear a packet	
	def clearPacket(self):
		self.magicNumber 		= self.RCI_MAGIC_NUMBER
		self.majorVersion 		= self.RCI_MAJOR_VERSION
		self.minorVersion 		= self.RCI_MINOR_VERSION
		self.packetDescriptor	= 0
		self.timeStamp			= 0
		self.numBytesPayload	= 0
		self.payload			= array('c')
		
		
	
		
