from RciLog import RciLog
from xml.etree.ElementTree import Element,SubElement
import codecs

class XmlElementList():
      
    log=RciLog("XmlElementList")

    def __init__(self):
        self.NodeList=[]
        self.NodesNumber = 0
    
    def getNodeByIndex(self,nodeIndex): 
        return self.NodeList[nodeIndex]
    
    def getNodeByAttribute(self,attribute,attributeValue):
        for elt in self.NodeList: 
            pairsAttrValue = elt.xmlElement.items()
            if pairsAttrValue != None : 
                for pair in pairsAttrValue:
                    if attribute== pair[0] and attributeValue == pair[1]:
                        return elt
        return None
    
    def has(self,eltIndex):
        return eltIndex >= 0 and eltIndex < len(self.NodeList)

    def has_attr(self,attribute,attributeValue):
        return self.getNodeByAttribute(attribute,attributeValue) != None


       
class XmlElement():
    
    XML_CHAR_ENCODING = 'utf-8'

    XML_ROOT_ELEMENT= "rciroot"
     
    log=RciLog("XmlElement")
               
    def __init__(self,elt):
        self.xmlElement=elt
    
    def getChild(self,elementName):
        try :
            child=self.xmlElement.find(elementName)
            if child == None :
                self.log.write("No element found with name " + elementName ,'ERROR')
                return None
            else:
                self.log.write("I found a child named "+ elementName, 'DEBUG')
                return XmlElement(child) 
        except Exception as e :
            self.log.write("getChild : Catch exception:",'ERROR')
            print type(e)
            raise e

    def getChildren(self,elementName):
        children=self.xmlElement.findall(elementName)
        nbreChildren=len(children)
        if nbreChildren == 0 :
            self.log.write("No element found with name " + elementName , 'NOTIFICATION')
            return None
        # Wrap to Xml List node
        childrenList=XmlElementList()
        for i in range(nbreChildren):
            childrenList.NodeList.append(XmlElement(children[i]))
        childrenList.NodesNumber=nbreChildren
        return childrenList
    
    def insertChild(self,childName):
        child=SubElement(self.xmlElement, childName)
        return XmlElement(child)
    
    def insertChildPosition(self,childName,position):
        child = SubElement(self.xmlElement, childName).insert(position,self.xmlElement)
        return XmlElement(child)
    
    def removeChild(self,child):
        if self.xmlElement.find(child):
            self.xmlElement.remove(child)
            return True
        return False

    def getAttrValue(self,attribute):
        return self.xmlElement.get(attribute)

    def setAttrValue(self,attribute,value):
        if attribute in self.xmlElement.keys():
            self.xmlElement.set(attribute,value)
            return True
        return False

    def getChildTextContent(self,element):
        return self.xmlElement.findtext(element)

    def getTextContent(self):
        return self.xmlElement.text

    def setTextContent(self,text):
        self.xmlElement.text=text
            
   